@@ -12,9 +12,32 @@ use constant EUI48LENGTHDEC => 6;
 use constant EUI64LENGTHHEX => 16;
 use constant EUI64LENGTHDEC => 8;
 
+use constant ETHER2TOKEN => (
+## see also http://www-01.ibm.com/support/docview.wss?uid=nas114157020a771b25d862567250003b62c
+## note this table is rotated compared to the above link,
+## so that the hex values line up as a linear array :)
+## 0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f
+qw(00 80 40 c0 20 a0 60 e0 10 90 50 d0 30 b0 70 f0), # 0
+qw(08 88 48 c8 28 a8 68 e8 18 98 58 d8 38 b8 78 f8), # 1
+qw(04 84 44 c4 24 a4 64 e4 14 94 54 d4 34 b4 74 f4), # 2
+qw(0c 8c 4c cc 2c ac 6c ec 1c 9c 5c dc 3c bc 7c fc), # 3
+qw(02 82 42 c2 22 a2 62 e2 12 92 52 d2 32 b2 72 f2), # 4
+qw(0a 8a 4a ca 2a aa 6a ea 1a 9a 5a da 3a ba 7a fa), # 5
+qw(06 86 46 c6 26 a6 66 e6 16 96 56 d6 36 b6 76 f6), # 6
+qw(0e 8e 4e ce 2e ae 6e ee 1e 9e 5e de 3e be 7e fe), # 7
+qw(01 81 41 c1 21 a1 61 e1 11 91 51 d1 31 b1 71 f1), # 8
+qw(09 89 49 c9 29 a9 69 e9 19 99 59 d9 39 b9 79 f9), # 9
+qw(05 85 45 c5 25 a5 65 e5 15 95 55 d5 35 b5 75 f5), # a
+qw(0d 8d 4d cd 2d ad 6d ed 1d 9d 5d dd 3d bd 7d fd), # b
+qw(03 83 43 c3 23 a3 63 e3 13 93 53 d3 33 b3 73 f3), # c
+qw(0b 8b 4b cb 2b ab 6b eb 1b 9b 5b db 3b bb 7b fb), # d
+qw(07 87 47 c7 27 a7 67 e7 17 97 57 d7 37 b7 77 f7), # e
+qw(0f 8f 4f cf 2f af 6f ef 1f 9f 5f df 3f bf 7f ff), # f
+);
+
 use base qw( Exporter );
 use vars qw( $VERSION %EXPORT_TAGS @EXPORT_OK );
-$VERSION = (qw$Revision: 0.72 $)[1];
+$VERSION = (qw$Revision: 0.73 $)[1];
 
 %EXPORT_TAGS = (
     all => [
@@ -26,21 +49,23 @@ $VERSION = (qw$Revision: 0.72 $)[1];
           mac_as_microsoft mac_as_cisco
           mac_as_bpr       mac_as_ieee
           mac_as_ipv6_suffix
+          mac_as_tokenring
         )
     ],
     properties => [
-    	qw(
+        qw(
           mac_is_eui48     mac_is_eui64
           mac_is_unicast   mac_is_multicast
           mac_is_local     mac_is_universal
-    	)
+        )
     ],
     normals => [
-    	qw(
+        qw(
           mac_as_basic     mac_as_sun
           mac_as_microsoft mac_as_cisco
           mac_as_bpr       mac_as_ieee
           mac_as_ipv6_suffix
+          mac_as_tokenring
         )
     ],
 );
@@ -70,11 +95,13 @@ OIE::Utils::MAC - Handles hardware MAC Addresses (EUI-48 and EUI-64)
     print "Universally Administered\n" if $mac->is_universal;
 
     print 'Basic Format: ',$mac->as_basic,"\n";
-    print 'Bpr Format: ',$mac->as_bpr,"\n";
-    print 'Cisco Format: '$mac->as_cisco,"\n";
-    print 'IEEE Format: '$mac->as_ieee,"\n";
+    print 'Bpr Format: ',  $mac->as_bpr,"\n";
+    print 'Cisco Format: ',$mac->as_cisco,"\n";
+    print 'IEEE Format: ', $mac->as_ieee,"\n";
+    print 'IPv6 Address: ',$mac->as_ipv6_suffix,"\n";
     print 'Microsoft Format: ',$mac->as_microsoft,"\n";
-    print 'Sun Format: ',$mac->as_sun,"\n";
+    print 'Sun Format: ',  $mac->as_sun,"\n";
+    print 'Token Ring Format: ', $mac->as_tokenring,"\n";
 
 
     use OIE::Utils::MAC qw( :all );
@@ -91,11 +118,14 @@ OIE::Utils::MAC - Handles hardware MAC Addresses (EUI-48 and EUI-64)
     print "Universally Administered\n" if mac_is_universal($mac);
 
     print 'Basic Format: ',mac_as_basic($mac),"\n";
-    print 'Bpr Format: ',mac_as_bpr($mac),"\n";
-    print 'Cisco Format: 'mac_as_cisco($mac),"\n";
-    print 'IEEE Format: 'mac_as_ieee($mac),"\n";
+    print 'Bpr Format: ',  mac_as_bpr($mac),"\n";
+    print 'Cisco Format: ',mac_as_cisco($mac),"\n";
+    print 'IEEE Format: ', mac_as_ieee($mac),"\n";
+    print 'IPv6 Address: ',mac_as_ipv6_suffix($mac),"\n";
     print 'Microsoft Format: ',mac_as_microsoft($mac),"\n";
-    print 'Sun Format: ',mac_as_sun($mac),"\n";
+    print 'Sun Format: ',  mac_as_sun($mac),"\n";
+    print 'Token Ring Format: ',mac_as_tokenring($mac),"\n";
+
 
 =head1 DESCRIPTION
 
@@ -407,17 +437,16 @@ sub as_sun {
 =head2 as_tokenring
 
 returns the mac address normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character) and each octect is bit-reversed order
+(ie after every 2nd character) and each octect is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
 
  00-2d-6a-1e-59-3d
 
 =cut
 
 sub as_tokenring {
-    croak 'not yet implemented';
 
-    # my $self = shift;
-    # return join( q{-}, map { sprintf( '%01x', $_ ) } @{ $self->{mac} } );
+    my $self = shift;
+    return join( q{-}, map { (ETHER2TOKEN)[$_] } @{ $self->{mac} } );
 }
 
 =head2 to_eui48
@@ -716,7 +745,7 @@ sub mac_as_sun {
 =head2 mac_as_tokenring($mac)
 
 returns the mac address in $mac normalized as a hexidecimal string that is 0 padded and with B<-> delimiting every octet
-(ie after every 2nd character) and each octect is bit-reversed order
+(ie after every 2nd character) and each octect is bit-reversed order. So 10 00 5A 4D BC 96 becomes 08 00 5A B2 3D 69.
 
  00-2d-6a-1e-59-3d
 
@@ -724,35 +753,28 @@ returns the mac address in $mac normalized as a hexidecimal string that is 0 pad
 
 sub mac_as_tokenring {
 
-    croak 'not yet implemented';
+    my $mac = shift;
+
+    croak 'please use as_tokenring'
+      if ref $mac eq __PACKAGE__;
+    croak 'argument must be a string' if ref $mac;
+
+    return as_tokenring( { mac => _mac_to_integers($mac) } )
 
 }
 
 =head1 CREDITS
 
-Myself, Dean Hamstead C<< <dean.hamstead@fragfest.com.au> >>
-
-Stolen lots of ideas and some pod content from L<Device::MAC> and L<Net::MAC>.
-
-Various contributers thanks to GitHub
-
-=head1 DEVELOPMENT
-
-Please fork this and send me pull requests! L<< https://github.com/djzort/Perl-NetAddr-MAC >>
+Stolen lots of ideas and some pod content from L<Device::MAC> and L<Net::MAC>
 
 =head1 TODO
 
  - tests!
  - as_tokenring - need to find a nifty way to reverse the bit order
 
-=head1 COPYRIGHT
+=head1 AUTHOR
 
-Copyright (c) 2011, Dean Hamstead
-
-=head1 LICENSE
-
-This library is free software. You can redistribute it and/or modify it under
-the same terms as Perl itself.
+Dean Hamstead C<< <dean.hamstead@optusnet.com.au> >>
 
 =cut
 
